/*
 * File:		Winamp.c
 *
 * Purpose:		Sample Plug-In for ATI Remote Wonder. Used to control
 *				Winamp 2x.
 *
 * Author:		ATI with modifications by NiDan
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 *
 * Use of this Software Development Kit is subject to the terms and conditions
 * of the ATI Technologies Inc. Software Development Kit License Agreement for 
 * the "REMOTE WONDER Plug-In SDK". If you have not accepted and agreed to this 
 * License, you have no rights to use the software contained herein.  
 */

#include <windows.h>
#include "ammo.h"				// Required for ATI Remote Wonder plug-in SDK API
#include "frontend.h"			// Required for Winamp message definitions
#include <stdio.h>
#include <string.h>


// The number of mapped programmable functions provided by the plug-in
#define kNumFunctions				(36)

// List of mapped functions for the programmable keys
#define kTogglesTitleAutoscroll		(0)
#define kTogglesAlwaysOnTop			(1)
#define kTogglesWindowshade			(2)
#define kTogglesPlaylistWindowshade	(3)
#define kTogglesEQ					(4)
#define kTogglesMainWindow			(5)
#define kTogglesMiniBrowser			(6)
#define kTogglesRepeat				(7)
#define kTogglesShuffle				(8)
#define kTogglesEasyMove			(9)
#define kEQLoadsPreset				(10)
#define kEQSavesPreset				(11)
#define kEQOpensLoadPresetDialog	(12)
#define kEQOpensAutoloadDialog		(13)
#define kEQLoadsDefaultPreset		(14)
#define kEQOpensSavePresetDialog	(15)
#define kEQOpensAutoloadSaveDialog	(16)
#define kEQOpensDeletePresetDialog	(17)
#define kEQOpensDeleteAndAutoload	(18)
#define kSetsTimeElapsed			(19)
#define kSetsTimeRemaining			(20)
#define kSkipsForward				(21)
#define kSkipsBackward				(22)
#define kLoadsBookmarkList			(23)
#define kExecutesVisualization		(24)
#define kGoesToStartOfPlaylist		(25)
#define kGoesBackInPlaylist			(26)
#define kGoesToEndOfPlaylist		(27)
#define kOpensFileDialog			(28)
#define kOpensURLDialog				(29)
#define kOpensJumpToTimeDialog		(30)
#define kOpensJumpToFileDialog		(31)
#define kStopsAfterCurrentTrack		(32)
#define kOpensFileInfo				(33)
#define kFadesAndStop				(34)
#define kConfiguresVisualization	(35)

#define IDD_CONFIGURE 101 
#define ID_FOCUS_BOX  102 
#define ID_STOPWATCH  103
#define ID_RESIZE     104
#define ID_VOLUME     107
#define ID_VOLINC	  1020
#define ID_VER		  1025
#define BUFSIZE       80

FILE *fp;
int gFocusBoxState,gStopWatch,gResize,gVolume,gVolInc,gVer;
HINSTANCE g_hInstance;
char szRemDir[BUFSIZE],szVer[10];

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved ) 
{ 
	HKEY hKey;
	DWORD dwBuf=BUFSIZE;

    if( dwReason==DLL_PROCESS_ATTACH ) { 
        g_hInstance = hInstance; 
    } 

	RegOpenKeyEx( HKEY_LOCAL_MACHINE,"SOFTWARE\\ATI Technologies\\Multimedia\\Remote Wonder", 0, KEY_QUERY_VALUE, &hKey );
	RegQueryValueEx( hKey, "PluginsPath", NULL, NULL, (LPBYTE) szRemDir, &dwBuf);
    RegCloseKey( hKey );
	strcat(szRemDir,"\\Winamp.ini");

	if ((gFocusBoxState!=1)||(gFocusBoxState!=0))
	{
		fp=fopen(szRemDir,"r");
		if(fp==NULL)
			fp=fopen(szRemDir,"ar");
		fscanf(fp,"%i",&gVer);
		fscanf(fp,"%i",&gFocusBoxState);
		fscanf(fp,"%i",&gStopWatch);
		fscanf(fp,"%i",&gResize);
		fscanf(fp,"%i",&gVolume);
		fscanf(fp,"%i",&gVolInc);
		fclose(fp);
		if(gVolInc<1)
			gVolInc=1;
	}
    return TRUE;     
} 

// List of strings for the programmable keys (A-F). Each index in this list
// is mapped to a function in the defintions above.
static char *functions[kNumFunctions] = 
{	
	// To create groups in the tree control use the "|" symbol to seperate
	// the branch from the root. For Winamp, we have three types of groups.
	// The general options are put into the root while settings that are
	// toggled and equalizer settings each have a seperate branch.

	// Group the toggle settings into a seperate branch.
	"Toggles|Title autoscrolling",
	"Toggles|Always on top",
	"Toggles|Windowshade",
	"Toggles|Playlist windowshade",
	"Toggles|Equalizer window",
	"Toggles|Main window",
	"Toggles|Minibrowser window",
	"Toggles|Repeat",
	"Toggles|Shuffle",
	"Toggles|Easymove",

	// Group the equalizer settings into a seperate branch.
	"Equalizer Settings|Loads a preset from an equalizer file",
	"Equalizer Settings|Saves a preset to an equalizer file",
	"Equalizer Settings|Opens load preset dialog",
	"Equalizer Settings|Opens auto-load preset dialog",
	"Equalizer Settings|Loads default preset",
	"Equalizer Settings|Opens save preset dialog",
	"Equalizer Settings|Opens save auto-load preset dialog",
	"Equalizer Settings|Opens delete preset dialog",
	"Equalizer Settings|Opens delete auto load preset dialog",

	// All of the general options are at the root level.
	"Sets time display mode to elapsed",
	"Sets time display mode to remaining",
	"Skips forward 5 seconds",
	"Skips backward 5 seconds",	
	"Loads bookmark list",
	"Executes current visualization",
	"Goes to start of playlist",
	"Goes back 10 tracks in playlist",
	"Goes to end of playlist",
	"Opens file dialog",
	"Opens URL dialog",
	"Opens jump to time dialog",
	"Opens jump to file dialog",
	"Stops after current track",
	"Opens file information dialog",
	"Fades out and stops",
	"Configures current visualization"
};

// Function prototype to a function that handles programmed key events.
static BOOL HandleProgrammedKey (HWND hWinamp, WORD wKeyEvent, WORD wState);

/*
 * Function:	WhatKeysDoYouWant
 *
 * Purpose:		Determines what groups of keys should be re-directed to
 *				the plug-in.
 *
 * Inputs:		None
 *
 * Returns:		A bit mask which indicates the groups of keys that should
 *				be re-directed to this plug-in.
 *
 * When called: This function is called once when the plug-in is first loaded,
 *				and again after calling Configure().
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
DWORD WhatKeysDoYouWant (void)
{
	// For winamp we need to control the following keys:
	//
	// CUSTOM_MAPPED - provide list of programmable options
	// VOLUME_GROUP - adjust the playback volume
	// PLAY_GROUP - play, pause, stop, FF, RW
	// SETUP - toggle the preferences dialog
	// MENU - toggle the playlist editor
	// RESIZE - toggle doublesize mode or toggle repeat
	// STOPWATCH - add current track as a bookmark or toggle shuffle 

	return (CUSTOM_MAPPED|VOLUME_GROUP|PLAY_GROUP|SETUP|MENU|RESIZE|STOPWATCH);
}


/*
 * Function:	EnumerateProgrammableFunction
 *
 * Purpose:		Enumerate the descriptions of the programamble
 *				functions provided by this plug-in.
 *
 * Inputs:		wIndex - index of requested description
 *
 * Returns:		Pointer to the programmable function description
 *				or NULL if wIndex is greater than the number of
 *				programmable functions provided by this plug-in.
 *
 * When called:	This function is called after any call to WhatKeysDoYouWant()
 *				that returns a request for custom mapped keys (CUSTOM_MAPPED).
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
char *EnumerateProgrammableFunction (WORD wIndex)
{	
	if (wIndex >= kNumFunctions) 
		return NULL;

	// Return a pointer to the requested description (see table above)
	return functions[wIndex];
}

/* Function: ConfigureDlgProc 
 * 
 * Purpose: Handles the Configure dialog box commands 
 * 
 * Inputs: hDlg - handle of configure dialog 
 * Message - message to handle 
 * wParam - first message parameter 
 * lParam - second message parameter 
 * 
 * Returns: TRUE if the message was handled, FALSE otherwise. 
*/ 
BOOL CALLBACK ConfigureDlgProc (HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam) 
{ 
HWND hWnd;
int gIndex1,gIndex2; 

switch (Message) 
{
case WM_INITDIALOG:
	if(gFocusBoxState==1)
		SendDlgItemMessage(hDlg, ID_FOCUS_BOX, BM_SETCHECK, BST_CHECKED, 0);
	
	if(gStopWatch==1)
		SendDlgItemMessage(hDlg, ID_STOPWATCH, BM_SETCHECK, BST_CHECKED, 0);

	if(gResize==1)
		SendDlgItemMessage(hDlg, ID_RESIZE, BM_SETCHECK, BST_CHECKED, 0);

	if(gVolume==1)
		SendDlgItemMessage(hDlg, ID_VOLUME, BM_SETCHECK, BST_CHECKED, 0);
	SetDlgItemInt(hDlg, ID_VOLINC, gVolInc, FALSE);
	gIndex1 = SendDlgItemMessage(hDlg, ID_VER, CB_ADDSTRING, 0, (LPARAM)"Winamp 2.x");
	gIndex2 = SendDlgItemMessage(hDlg, ID_VER, CB_ADDSTRING, 0, (LPARAM)"Winamp 3");
	if(gVer==2)
		SendDlgItemMessage(hDlg, ID_VER, CB_SETCURSEL, gIndex1, 0);
	else
		SendDlgItemMessage(hDlg, ID_VER, CB_SETCURSEL, gIndex2, 0);


case WM_COMMAND: 
// Close the dialog if OK or CANCEL selected 
	if (LOWORD(wParam) == IDOK) 
	{	 
		hWnd = GetDlgItem(hDlg,ID_FOCUS_BOX); 
		gFocusBoxState = 0; 
		if (SendMessage(hWnd, BM_GETCHECK, 0, 0)==BST_CHECKED) 
				gFocusBoxState = 1; 
		
		hWnd = GetDlgItem(hDlg,ID_STOPWATCH); 
		gStopWatch = 0; 
		if (SendMessage(hWnd, BM_GETCHECK, 0, 0)==BST_CHECKED) 
				gStopWatch = 1; 

		hWnd = GetDlgItem(hDlg,ID_RESIZE); 
		gResize = 0; 
		if (SendMessage(hWnd, BM_GETCHECK, 0, 0)==BST_CHECKED) 
				gResize = 1; 

		hWnd = GetDlgItem(hDlg,ID_VOLUME); 
		gVolume = 0; 
		if (SendMessage(hWnd, BM_GETCHECK, 0, 0)==BST_CHECKED) 
				gVolume = 1; 
		
		gVolInc = GetDlgItemInt(hDlg, ID_VOLINC, 0, FALSE);
		if(gVolInc>100)
			gVolInc=100;
		else if (gVolInc<1)
				gVolInc=1;

		GetDlgItemText(hDlg, ID_VER, szVer, 10);
		if(strlen(szVer)>8)
			gVer=2;
		else
			gVer=3;

		EndDialog(hDlg, LOWORD(wParam));
		return TRUE; 
	} 
	if (LOWORD(wParam) == IDCANCEL) 
	{ 
		EndDialog(hDlg, LOWORD(wParam)); 
		return TRUE; 
	} 
}; 
return FALSE; 
} 

/* 
 * Function: Configure 
 * 
 * Purpose: Displays any configuration/setup page required by 
 * this plug-in. 
 * 
 * Inputs: hWnd - handle to the parent window. The parent window 
 * is used to ensure correct modal behaviour when creating 
 * dialog and message boxes in this plug-in. 
 * 
 * Returns: 	None
 *
 * When called: This function is called when the "Configure" button in the
 *				ATI Remote Wonder software is pressed.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
*/ 
void Configure (HANDLE hWnd) 
{ 
FARPROC dlgConfigure; 

dlgConfigure = MakeProcInstance((FARPROC) ConfigureDlgProc, g_hInstance);
DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_CONFIGURE), (HWND) hWnd, (DLGPROC) dlgConfigure);
if(gFocusBoxState==1)
FreeProcInstance(dlgConfigure);

fp=fopen(szRemDir,"w");
fprintf(fp,"%i\n",gVer);
fprintf(fp,"%i\n",gFocusBoxState);
fprintf(fp,"%i\n",gStopWatch);
fprintf(fp,"%i\n",gResize);
fprintf(fp,"%i\n",gVolume);
fprintf(fp,"%i\n",gVolInc);
fclose(fp);
} 

/*
 * Function:	AreYouInFocus
 *
 * Purpose:		Determines if this plug-in is currently in-focus.
 *
 * Inputs:		None
 *
 * Returns:		TRUE if the plug-in is currently in focus, FALSE
 *				otherwise.
 *
 * When called:	This function is called when a key event occurs that
 *				the plug-in has requested.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
int AreYouInFocus (void)
{
	// The winamp main window uses a class name of 'Winamp v1.x'. Winamp
	// 2.X uses this name for compatibility with previous versions.
	
	HWND hWinamp;
	HWND hForeground = GetForegroundWindow();

	if(gVer == 2)
		hWinamp = FindWindow("Winamp v1.x", NULL);
	else if (gVer == 3)
		hWinamp = FindWindow("STUDIO", NULL);


	if (hWinamp == NULL)
	{
		// Winamp is not running
		return FALSE;
	}

	if (hWinamp == hForeground)
	{
		// Winamp main window is in focus
		return TRUE;
	}
	
	if (hWinamp == GetWindow(hForeground, GW_OWNER))
	{
		// EQ, Minibrowser, Playlist or another Winamp window is in focus
		return TRUE;
	}

	// Winamp is running but is not in focus
	if(gFocusBoxState==1)
		return TRUE;
	return FALSE;
}


/*
 * Function:	HandleKey
 *
 * Purpose:		Handles remote control keys.
 *
 * Inputs:		bCustom   - TRUE if wKeyEvent has been mapped to a custom
 *						    programamble option.
 *				wKeyEvent - Remote control key code for button that was
 *							pressed.
 *				wState    - RMCTRL_KEY_ON, when button is first pressed
 *						    RMCTRL_KEY_REPEAT, when button is held down
 *						    RMCTRL_KEY_OFF, when button is released
 *
 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.
 *
 * When called: This function is called whenever a key event occurs that
 *				the plug-in has requested when the plug-in is in focus.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
BOOL HandleKey (BOOL bCustom, WORD wKeyEvent, WORD wState)
{
	// Get the Winamp main window.
	HWND hWinamp;
	int z;

	if(gVer == 2)
		hWinamp = FindWindow("Winamp v1.x", NULL);
	else if (gVer == 3)
		hWinamp = FindWindow("STUDIO", NULL);

	// If the main window is not found Winamp has been closed.
	if (hWinamp == NULL)
	{
		return FALSE;
	}
	
	// Handle programmed keys seperately
	if (bCustom)	
	{
		return HandleProgrammedKey(hWinamp, wKeyEvent, wState);	
	}

	// Handle non-programmed keys
	if (wState == RMCTRL_KEY_ON)
	{
		// The Winamp SDK (defined in frontend.h) does not have a complete
		// set of definitions for all supported messages. As a result some
		// of the messages sent to the Winamp main window are constants.

		switch (wKeyEvent)
		{
		case RMCTRL_STOPWATCH:									 // bookmark/crossfade or shuffle
			// Use PostMessage to send a command to the Winamp window. For the remote
			// control to remain responsive HandleKey must return quickly. If you
			// use SendMessage here and Winamp opens a dialog HandleKey will not
			// return until the dialog is closed. 
			if(gVer==2)
			{
				if(gStopWatch==1)
					PostMessage(hWinamp, WM_COMMAND, 40023, 0);
				else
					PostMessage(hWinamp, WM_COMMAND, 40321, 0);
			}
			else
			{
				if(gStopWatch==1)
				{
					PostMessage(hWinamp, WM_KEYDOWN, 0x53, 0);
					PostMessage(hWinamp, WM_KEYUP, 0x53, 0);
				}
				else
				{
					PostMessage(hWinamp, WM_KEYDOWN, 0x46, 0);  // wa3 = crossfade
					PostMessage(hWinamp, WM_KEYUP, 0x46, 0);
				}
			}
			return TRUE;

		case RMCTRL_RESIZE:										// resize or repeat
			if(gVer==2)
			{
				if(gResize==1)
					PostMessage(hWinamp, WM_COMMAND, 40022, 0);
				else
					PostMessage(hWinamp, WM_COMMAND, 40165, 0);
			}
			else
			{
				if(gResize==1)
				{
					PostMessage(hWinamp, WM_KEYDOWN, 0x52, 0);
					PostMessage(hWinamp, WM_KEYUP, 0x52, 0);
				}
				else
				{
					PostMessage(hWinamp, WM_KEYDOWN, VK_CONTROL, 0);
					PostMessage(hWinamp, WM_KEYDOWN, 0x44, 0);
					PostMessage(hWinamp, WM_KEYUP, 0x44, 0);
					PostMessage(hWinamp, WM_KEYUP, VK_CONTROL, 0);
				}
			}
			return TRUE;

		case RMCTRL_MENU:
			PostMessage(hWinamp, WM_COMMAND, WINAMP_OPTIONS_PLEDIT, 0);
			return TRUE;

		case RMCTRL_SETUP:
			if(gVer==2)
			{
				PostMessage(hWinamp, WM_COMMAND, WINAMP_OPTIONS_PREFS, 0);
			}
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, VK_CONTROL, 0);
				PostMessage(hWinamp, WM_KEYDOWN, 0x50, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x50, 0);
				PostMessage(hWinamp, WM_KEYUP, VK_CONTROL, 0);
			}
			return TRUE;

		case RMCTRL_VOLUMEUP:
			if(gVer==2)
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
						PostMessage(hWinamp, WM_COMMAND, WINAMP_VOLUMEUP, 0);
					return TRUE;
				}
				else return FALSE;
			}
			else
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
					{
						PostMessage(hWinamp, WM_KEYDOWN, VK_UP, 0);
						PostMessage(hWinamp, WM_KEYUP, VK_UP, 0);
					}
					return TRUE;	
				}
				else return FALSE;
			}

	
		case RMCTRL_VOLUMEDOWN:
			if(gVer==2)
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
						PostMessage(hWinamp, WM_COMMAND, WINAMP_VOLUMEDOWN, 0);
					return TRUE;	
				}
				else return FALSE;
			}
			else
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
					{
						PostMessage(hWinamp, WM_KEYDOWN, VK_DOWN, 0);
						PostMessage(hWinamp, WM_KEYUP, VK_DOWN, 0);
					}
					return TRUE;	
				}
				else return FALSE;
			}

		case RMCTRL_MUTE:
			if(gVolume==1)
			{
				PostMessage(hWinamp, WM_WA_IPC, 0, IPC_SETVOLUME);
				return TRUE;
			}
			else return FALSE;

		case RMCTRL_PLAY:
			if(gVer==2)
				PostMessage(hWinamp, WM_WA_IPC, 0, IPC_STARTPLAY);
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, 0x58, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x58, 0);
			}
			return TRUE;	

		case RMCTRL_STOP:
			if(gVer==2)
				PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON4, 0);
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, 0x56, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x56, 0);
			}
			return TRUE;

		case RMCTRL_PAUSE:
			if(gVer==2)
				PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON3, 0);
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, 0x43, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x43, 0);
			}
			return TRUE;

		case RMCTRL_FF:
			if(gVer==2)
				PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON5, 0);
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, 0x42, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x42, 0);
			}
			return TRUE;

		case RMCTRL_RW:
			if(gVer==2)
				PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON1, 0);
			else
			{
				PostMessage(hWinamp, WM_KEYDOWN, 0x5A, 0);
				PostMessage(hWinamp, WM_KEYUP, 0x5A, 0);
			}
			return TRUE;
		}
	}
	else if (wState == RMCTRL_KEY_REPEAT)	
	{
		switch (wKeyEvent)
		{
		case RMCTRL_VOLUMEUP:
			if(gVer==2)
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
						PostMessage(hWinamp, WM_COMMAND, WINAMP_VOLUMEUP, 0);
					return TRUE;
				}
				else return FALSE;
			}
			else
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
					{
						PostMessage(hWinamp, WM_KEYDOWN, VK_UP, 0);
						PostMessage(hWinamp, WM_KEYUP, VK_UP, 0);
					}
					return TRUE;	
				}
				else return FALSE;
			}
			
		case RMCTRL_VOLUMEDOWN:
			if(gVer==2)
			{
				if(gVolume == 1)
				{
					for(z=0;z<gVolInc;z++)
						PostMessage(hWinamp, WM_COMMAND, WINAMP_VOLUMEDOWN, 0);
					return TRUE;	
				}
				else return FALSE;
			}
			else
			{
				if(gVolume==1)
				{
					for(z=0;z<gVolInc;z++)
					{
						PostMessage(hWinamp, WM_KEYDOWN, VK_DOWN, 0);
						PostMessage(hWinamp, WM_KEYUP, VK_DOWN, 0);
					}
					return TRUE;	
				}
				else return FALSE;
			}
		}			
	}

	// The plug-in did not process this key. It is important to return FALSE
	// if the key was not processed so that other plug-ins or the ATI Remote Wonder
	// software can handle the key press.
	return FALSE;
}


/*
 * Function:	HandleProgrammedKey
 *
 * Purpose:		Handles remote control programmed keys
 *
 * Inputs:		hWinamp	  - Handle to the Winamp main window
 *				wKeyEvent - Remote control key code for button that was
 *							pressed.
 *				wState    - RMCTRL_KEY_ON, when button is first pressed
 *						    RMCTRL_KEY_REPEAT, when button is held down
 *						    RMCTRL_KEY_OFF, when button is released	
 *
 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.
 *
 * When called:	This function is called when a custom mapped programmable
 *				key has been pressed and the plug-in is in focus. This is
 *				a helper function for HandleKey().
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
static BOOL HandleProgrammedKey (HWND hWinamp, WORD wKeyEvent, WORD wState)
{
	HWND hEQ;

	if (wState != RMCTRL_KEY_ON)
		return FALSE;

	// Get the Winamp equalizer window
	hEQ = FindWindow("Winamp EQ", NULL);			

	switch (wKeyEvent)
	{		
	case kTogglesTitleAutoscroll:
		PostMessage(hWinamp, WM_COMMAND, 40189, 0);
		return TRUE;
	case kTogglesAlwaysOnTop:
		PostMessage(hWinamp, WM_COMMAND, 40019, 0);
		return TRUE;
	case kTogglesWindowshade:
		PostMessage(hWinamp, WM_COMMAND, 40064, 0);
		return TRUE;
	case kTogglesPlaylistWindowshade:
		PostMessage(hWinamp, WM_COMMAND, 40266, 0);
		return TRUE;
	case kTogglesEQ:
		PostMessage(hWinamp, WM_COMMAND, 40036, 0);
		return TRUE;
	case kTogglesMainWindow:
		PostMessage(hWinamp, WM_COMMAND, 40258, 0);
		return TRUE;
	case kTogglesMiniBrowser:
		PostMessage(hWinamp, WM_COMMAND, 40298, 0);
		return TRUE;
	case kTogglesRepeat:
		PostMessage(hWinamp, WM_COMMAND, 40022, 0);
		return TRUE;
	case kTogglesShuffle:
		PostMessage(hWinamp, WM_COMMAND, 40023, 0);
		return TRUE;
	case kTogglesEasyMove:
		PostMessage(hWinamp, WM_COMMAND, 40186, 0);
		return TRUE;
	case kSetsTimeElapsed:
		PostMessage(hWinamp, WM_COMMAND, 40037, 0);
		return TRUE;
	case kSetsTimeRemaining:
		PostMessage(hWinamp, WM_COMMAND, 40038, 0);
		return TRUE;
	case kSkipsForward:
		PostMessage(hWinamp, WM_COMMAND, 40148, 0);
		return TRUE;
	case kSkipsBackward:
		PostMessage(hWinamp, WM_COMMAND, 40144, 0);
		return TRUE;
	case kExecutesVisualization:
		PostMessage(hWinamp, WM_COMMAND, 40192, 0);
		return TRUE;
	case kLoadsBookmarkList:
		PostMessage(hWinamp, WM_COMMAND, 40320, 0);
		return TRUE;
	case kGoesToStartOfPlaylist:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON1_CTRL, 0);
		return TRUE;
	case kGoesBackInPlaylist:
		PostMessage(hWinamp, WM_COMMAND, 40197, 0);
		return TRUE;
	case kGoesToEndOfPlaylist:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON5_CTRL, 0);
		return TRUE;
	case kOpensFileDialog:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_FILE_PLAY, 0);
		return TRUE;
	case kOpensURLDialog:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON2_CTRL, 0);
		return TRUE;
	case kOpensJumpToTimeDialog:
		PostMessage(hWinamp, WM_COMMAND, 40193, 0);
		return TRUE;
	case kOpensJumpToFileDialog:
		PostMessage(hWinamp, WM_COMMAND, 40194, 0);
		return TRUE;
	case kStopsAfterCurrentTrack:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON4_CTRL, 0);
		return TRUE;
	case kOpensFileInfo:
		PostMessage(hWinamp, WM_COMMAND, 40188, 0);
		return TRUE;
	case kFadesAndStop:
		PostMessage(hWinamp, WM_COMMAND, WINAMP_BUTTON4_SHIFT, 0);
		return TRUE;
	case kConfiguresVisualization:
		PostMessage(hWinamp, WM_COMMAND, 40221, 0);
		return TRUE;		
	case kEQLoadsPreset:
		PostMessage(hEQ, WM_COMMAND, 40253, 0);
		return TRUE;
	case kEQSavesPreset:
		PostMessage(hEQ, WM_COMMAND, 40254, 0);
		return TRUE;
	case kEQOpensLoadPresetDialog:
		PostMessage(hEQ, WM_COMMAND, 40172, 0);
		return TRUE;
	case kEQOpensAutoloadDialog:
		PostMessage(hEQ, WM_COMMAND, 40173, 0);
		return TRUE;
	case kEQLoadsDefaultPreset:
		PostMessage(hEQ, WM_COMMAND, 40174, 0);
		return TRUE;
	case kEQOpensSavePresetDialog:
		PostMessage(hEQ, WM_COMMAND, 40175, 0);
		return TRUE;
	case kEQOpensAutoloadSaveDialog:
		PostMessage(hEQ, WM_COMMAND, 40176, 0);
		return TRUE;
	case kEQOpensDeletePresetDialog:
		PostMessage(hEQ, WM_COMMAND, 40178, 0);
		return TRUE;
	case kEQOpensDeleteAndAutoload:
		PostMessage(hEQ, WM_COMMAND, 40180, 0);
		return TRUE;
	}

	// The programmed key was not processed
	return FALSE;
}